@@ -23,64 +23,48 @@ import tempfile
 import cliapp
 from vmdebootstrap.base import runcmd
 from lwr.vm import detect_kernels
-from lwr.apt_udeb import AptUdebDownloader
+from lwr.apt_udeb import get_apt_handler
 
 # pylint: disable=missing-docstring
 
-def generate_cfg(bootconfig):
+def generate_cfg(bootconfig, submenu=False):
     ret = str()
-    first = True
-    ret += "UI vesamenu.c32\n"
-    ret += "INCLUDE stdmenu.cfg\n"
+    if not submenu:
+        first = True
+        ret += "INCLUDE stdmenu.cfg\n"
+        ret += "MENU title Main Menu\n"
+    else:
+        first = False
+     
     for entry in bootconfig.entries:
-        if entry['type'] is not 'linux':
-            continue
         label = "%s" % (entry['description'],)
+        if entry['type'] is 'menu':
+           if entry['subentries'].is_empty(["menu", "linux", "linux16", "com32"]):
+               continue
+           ret += "MENU begin advanced\n"
+           ret += "MENU title %s\n" % (label,)
+           ret += generate_cfg(entry['subentries'], submenu=True) 
+           ret += " LABEL mainmenu \n "
+           ret += " MENU label Back\n "
+           ret += " MENU exit\n "
+           ret += " MENU end\n "
+          
+        # do not want to default to menus
         if first:
             ret += "DEFAULT %s\n" % (label,)
-        ret += "LABEL %s\n" % (label,)
-        ret += "  SAY \"Booting %s...\"\n" % (entry['description'],)
-        ret += "  LINUX %s\n" % (entry['kernel'],)
-        if entry.get('initrd') is not None:
-            ret += "  APPEND initrd=%s %s\n" % (entry['initrd'], entry.get('cmdline', ''),)
-        elif entry.get('cmdline') is not None:
-            ret += "  APPEND %s\n" % (entry['cmdline'],)
+            first = False
+        if entry['type'].startswith('linux') or entry['type'] is 'com32':
+            type = 'linux' if entry['type'].startswith('linux') else entry['type']
+            ret += "LABEL %s\n" % (label,)
+            ret += "  SAY \"Booting %s...\"\n" % (entry['description'],)
+            ret += "  %s %s\n" % (entry['type'], entry['kernel'],)
+            if entry.get('initrd') is not None:
+                ret += "  APPEND initrd=%s %s\n" % (entry['initrd'], entry.get('cmdline', ''),)
+            elif entry.get('cmdline') is not None:
+                ret += "  APPEND %s\n" % (entry['cmdline'],)
         ret += "\n"
-        first = False
     return ret
 
-def prepare_download(destdir, mirror, suite, architecture):
-    apt_handler = AptUdebDownloader(destdir)
-    apt_handler.mirror = mirror
-    apt_handler.architecture = architecture
-    apt_handler.suite = suite
-    apt_handler.components = ['main']
-    apt_handler.prepare_apt()
-    return apt_handler
-
-
-def install_memtest(cdroot, mirror, suite, architecture):
-    """
-    Download and unpack the memtest86+ package.
-    """
-    destdir = tempfile.mkdtemp()
-    handler = prepare_download(destdir, mirror, suite, architecture)
-    filename = handler.download_package('memtest86+', destdir)
-    # these files are put directly into cdroot/live
-    memtest86_file = 'memtest86+.bin'
-    if filename:
-        runcmd(['dpkg', '-x', filename, destdir])
-        shutil.copyfile(
-            os.path.join(destdir, "boot/%s" % memtest86_file),
-            "%s/%s" % (cdroot, memtest86_file))
-    else:
-        handler.clean_up_apt()
-        shutil.rmtree(destdir)
-        raise cliapp.AppException('Unable to download memtest86+')
-    handler.clean_up_apt()
-    shutil.rmtree(destdir)
-
-
 def install_isolinux(cdroot, mirror, suite, architecture, bootconfig):
     """
     Download and unpack the correct syslinux-common
@@ -89,7 +73,7 @@ def install_isolinux(cdroot, mirror, suite, architecture, bootconfig):
     This function puts all files into isolinux/
     """
     destdir = tempfile.mkdtemp()
-    handler = prepare_download(destdir, mirror, suite, architecture)
+    handler = get_apt_handler(destdir, mirror, suite, architecture)
     filename = handler.download_package('syslinux-common', destdir)
     # these files are put directly into cdroot/isolinux
     syslinux_files = [
@@ -125,3 +109,7 @@ def install_isolinux(cdroot, mirror, suite, architecture, bootconfig):
     cfg = generate_cfg(bootconfig)
     with open("%s/%s" % (cdroot, "menu.cfg"), "w") as cfgout:
         cfgout.write(cfg)
+
+    # Fix the menu display size in stdmeny.cfg (#861421)
+    runcmd(['sed', '-i', 's,menu rows 12,menu rows 8,g',
+                os.path.join(cdroot, 'stdmenu.cfg')])
